#
# spec file for package ispc
#
# Copyright (c) 2024 SUSE LLC
# Copyright (c) 2020-2023 LISA GmbH, Bingen, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global min_llvm_version 16
%global max_llvm_version 19.9
%define libname libispcrt1

# LLVM is build with OpenMP support only on 64bit archs and x86
%ifarch aarch64 ppc64 ppc64le %{ix86} x86_64
%bcond_without openmp_task_model
%else
%bcond_with openmp_task_model
%endif

Name:           ispc
Version:        1.25.3
Release:        1.4
Summary:        C-based SPMD programming language compiler
License:        BSD-3-Clause
Group:          Development/Languages/C and C++
URL:            https://ispc.github.io/
Source:         https://github.com/%{name}/%{name}/archive/v%{version}/v-%{version}.tar.gz#/%{name}-%{version}.tar.gz
#!BuildIgnore:  clang15
BuildRequires:  bison
BuildRequires:  cmake >= 3.13
BuildRequires:  flex
%if %{suse_version} < 1550
# Leap/SLE 15 does not understand boolean requires
BuildRequires:  cmake(Clang) >= 19
#!BuildConflicts: cmake(Clang) >= 20
%endif
BuildRequires:  (cmake(Clang) >= %{min_llvm_version} with cmake(Clang) =< %{max_llvm_version})
BuildRequires:  (cmake(LLVM)  >= %{min_llvm_version} with cmake(LLVM)  =< %{max_llvm_version})
%if %{with openmp_task_model}
BuildRequires:  (libomp-devel-provider >= %{min_llvm_version} with libomp-devel-provider  =< %{max_llvm_version})
%else
BuildRequires:  tbb-devel
%endif
BuildRequires:  ncurses-devel
BuildRequires:  pkgconfig(python3)
%ifarch x86_64
# x86_64 always includes x86 target support: https://github.com/ispc/ispc/issues/1865
BuildRequires:  glibc-devel-32bit glibc-32bit
%endif
# aarch32 requires LLVM with aarch64 target support, likewise for ix86
ExclusiveArch:  x86_64 aarch64
# require devel for now for backwards compatibility (until now, packages just BuildRequire: ispc)
Requires:       %{name}-devel

%description
A compiler for a variant of the C programming language, with extensions for
"single program, multiple data" (SPMD) programming.

%package -n %{libname}
Summary:        C-based SPMD programming language compiler library
Group:          System/Libraries
# Old library name violating SLPP, only dash or <none> allowed as separator
Conflicts:      libispcrt_1 <= 1.18.0

%description -n %{libname}
Libary for a variant of the C programming language, with extensions for
"single program, multiple data" (SPMD) programming.

%package devel
Summary:        Development files for ispc
Group:          Development/Libraries/C and C++
Requires:       %{libname} = %{version}
Requires:       %{name} = %{version}

%description    devel
This package contains the C++ header, symbolic links to the shared
libraries and cmake files for %{name}.  If you would like to develop
programs using %{name}, you will need to install %{name}-devel.

%prep
%autosetup -p1

# fix clang library modules for our clang 10 and above
sed -i 's|set(CLANG_LIBRARY_LIST .*)|set(CLANG_LIBRARY_LIST clang-cpp)|' CMakeLists.txt

# disable build of static library, https://github.com/ispc/ispc/issues/2385
sed -i -e '/build_ispcrt(STATIC/ s@.*@#\0@' ispcrt/CMakeLists.txt

%build
%define _lto_cflags "-flto=thin"
echo "optflags: %{optflags}"
%cmake \
        -DCMAKE_INSTALL_PREFIX=%{_prefix} \
        -DCMAKE_C_FLAGS:STRING="$CFLAGS %{optflags} -fPIE" \
        -DCMAKE_CXX_FLAGS:STRING="$CXXFLAGS %{optflags} -fPIE" \
        -DCMAKE_EXE_LINKER_FLAGS="%{optflags} -pie" \
        -DCURSES_CURSES_LIBRARY=/usr/%_lib/libncurses.so \
        -DISPCRT_BUILD_TASK_MODEL=%{?with_openmp_task_model:OpenMP}%{!?with_openmp_task_model:TBB} \
        -DISPC_INCLUDE_EXAMPLES=OFF \
        -DISPC_INCLUDE_TESTS=ON \
        -DISPCRT_BUILD_STATIC=OFF \
        %{nil}
%cmake_build

%install
%cmake_install

%check
pushd %__builddir
%cmake_build check-all

%post -n %{libname} -p /sbin/ldconfig
%postun -n %{libname} -p /sbin/ldconfig

%files -n %{libname}
%license LICENSE.txt
%doc *.md
%{_libdir}/*.so.*

%files
%license LICENSE.txt
%doc *.md
%{_bindir}/%{name}
%{_bindir}/check_isa

%files devel
%license LICENSE.txt
%{_includedir}/ispcrt
%{_libdir}/*.so
%{_libdir}/cmake/ispcrt-%{version}

%changelog
* Tue Nov 19 2024 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Update to 1.25.3
  https://github.com/ispc/ispc/releases/tag/v1.25.3
  https://github.com/ispc/ispc/releases/tag/v1.25.2
  https://github.com/ispc/ispc/releases/tag/v1.25.1
  https://github.com/ispc/ispc/releases/tag/v1.25.0
  https://github.com/ispc/ispc/releases/tag/v1.24.0
- Use LLVM 19
- Clean up spec file
* Mon Apr 29 2024 Marcus Rueckert <mrueckert@suse.de>
- Update to 1.23.0
  https://github.com/ispc/ispc/releases/tag/v1.23.0
  https://github.com/ispc/ispc/releases/tag/v1.22.0
  https://github.com/ispc/ispc/releases/tag/v1.21.1
  https://github.com/ispc/ispc/releases/tag/v1.21.0
  https://github.com/ispc/ispc/releases/tag/v1.20.0
  https://github.com/ispc/ispc/releases/tag/v1.19.0
- drop ispc-add-cstdint-include.patch:
  included in update
* Sun Mar 24 2024 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Replace libomp-devel requirement by libomp%%llvm_ver-devel to
  match llvm%%llvm_ver-devel. Needed because libomp-devel is now a
  metapackage that points to the latest version.
* Mon Nov  6 2023 Dominique Leuenberger <dimstar@opensuse.org>
- Fix build with RPM 4.19: unnumbered patches are no longer
  supported.
* Fri Mar 24 2023 Hans-Peter Jansen <hpj@urpla.net>
- Add ispc-add-cstdint-include.patch to fix compilation with GCC 13.
* Wed Nov 23 2022 Dominique Leuenberger <dimstar@opensuse.org>
- Update to version 1.18.1:
  + A minor ISPC update with security fix: zlib dependency was
    removed.
- Drop zlib-devel BuildRequires: no longer needed.
* Wed Oct 19 2022 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Enable aarch64 builds, disable (broken) aarch32
- Fix CMake config, remove static library from exported targets
- Enable tests
* Fri Oct  7 2022 Hans-Peter Jansen <hpj@urpla.net>
- Update to 1.18.0
  Check:
  https://github.com/ispc/ispc/blob/main/docs/ReleaseNotes.txt
- Package separate devel and library packages
- Specify a certain llvm version (llvm15 is supported from the
  next release only
- Adopt {n}curses support
- Make ispc depend on ispc-devel for backward compatibility
* Sun Apr 24 2022 Hans-Peter Jansen <hpj@urpla.net>
- Update to 1.17.0
  Check:
  https://github.com/ispc/ispc/blob/main/docs/ReleaseNotes.txt
- Switch to default clang/llvm build
* Tue Oct 12 2021 Hans-Peter Jansen <hpj@urpla.net>
- Update to 1.16.1
  Check:
  https://github.com/ispc/ispc/blob/main/docs/ReleaseNotes.txt
- Remove fix-llvm12-fallout.patch, upstreamed
- Explicitly use clang12 to build
* Sun May  2 2021 Hans-Peter Jansen <hpj@urpla.net>
- Add fix-llvm12-fallout.patch with some LLVM API changes backported
  as a temporary measure to fix LLVM 12 builds
* Sun Apr 18 2021 andy great <andythe_great@pm.me>
- Update to 1.15.0.
  * New loop unroll pragmas: #pragma unroll and #pragma nounroll
    directives provide loop unrolling optimization hints to the
    compiler. This pragma may be used immediately before a loop
    statement. Currently, this functionality is limited to uniform
    for and do-while.
  * More efficient packed_[load|store]_active() stdlib functions
    implementation (up to 2.5x faster), which now supports 64 bit types.
  * New cpus: icelake-server, tigerlake , alderlake, sapphirerapids.
  * Several stability fixes related to SOA types, bool varying type
    initialization, broken alignment information, type scoping.
  * Compile time improvements.
  * Initial support of ahead of time compilation to oneAPI Level Zero
    binary format using --emit-zebin switch. You can use this binary
    from ISPC Runtime by setting
  * ISPCRT_USE_ZEBIN env variable to 1. Please note that SPIR-V format
    is still a recommended and default way.
  * Initial function pointers implementation.
  * Global atomics support.
  * Double math functions support.
  * Memory functions support.
  * Reworked masking approach. We disabled genx hardware mask by
    default and use a software mask by default.
  * Improved address spaces differentiation.
  * Initial debug support.
  * TGLLP (TigerLake mobile CPU) support (--cpu=TGLLP).
* Fri Nov  6 2020 Hans-Peter Jansen <hpj@urpla.net>
  Note, we don't do any GPU builds yet, because
  - API is not stable
  - additional dependencies (that are not available, yet)
- Update to version v1.14.1:
  A minor ISPC update with a bug fix for AVX512 detection problem on macOS
  (for more details see issue #1854) and update of GPU version to use Level0 v1.0.
  CPU binaries are based on patched LLVM 10.0.1.
- Update to version v1.14.0:
  An ISPC release with several improvements for CPU and initial support of Intel
  graphics hardware architectures. The binaries in this release include CPU versions
  for Windows, Linux, and macOS, as previous releases, plus a GPU-enabled Linux binary,
  which supports both CPU and GPU. CPU binaries are based on patched LLVM 10.0.1.
  CPU changes include:
  + new avx2-i8x32, avx2-i16x16, avx512skx-i8x64, avx512skx-i16x32 targets.
    "generic" targets were removed.
  + several stability fixes, including bugs discovered during fuzzing ISPC by YARPGen.
  + integer division performance improvements.
  + support for __vectorcall calling convention on Windows x64 (enabled by
    '--vectorcall')
* Wed Jul  8 2020 Hans-Peter Jansen <hpj@urpla.net>
- fix build with 15.2
* Sun Apr 26 2020 Hans-Peter Jansen <hpj@urpla.net>
- Use lto option "thin"
* Sun Apr 26 2020 Hans-Peter Jansen <hpj@urpla.net>
- Switch to full llvm compilation (remove gcc enforcement)
* Sat Apr 25 2020 Hans-Peter Jansen <hpj@urpla.net>
- Fix llvm version dependency tweaking
- Exclude %%{ix86} for now
* Sat Apr 25 2020 Martin Pluskal <mpluskal@suse.com>
- Clarify cmake dependency
* Fri Apr 24 2020 Hans-Peter Jansen <hpj@urpla.net>
- Version 1.13.0: initial release
- Add patch 0001-Link-against-libclang-cpp.so.patch
- Add patch 0002-fix-compiler-flags.patch
- Conditional to build with older clang as well
